; ********************************************************************************
; RGB Stand-alone driver Functions include file

; Functions
; --------------------------------------------------------
;
;
;_pwmDrive      drive RGB LEDs with a PWM signal to control brightness
;               code loops continually.
;               Input on GPIO,4 determines whether active drive is high or low
;               The input is only read at power-on / reset.
;
; _seqData      Loads next data block for current sequence
;               resets to start at end of data
;               no parameters. contents of W lost
;
; _advSeq       Advance to next sequence.
;               Set Sequence No counter
;               Reset to first seqence when it reaches end
;               of available sequences.
;               no parameters.
;               return W=1, reset to first sequence
;               teturn W=0, not reset to first sequence
;
;_advResetBase  Initialise sequence data pointer variables
;               no parameters, contents of W lost
;               
;_sleep         If Sequence switch held down for over 1.2S 
;               put PIC to sleep
;               no parameters, PIC resets after return from sleep
;
;_initEEPROM    Initialise EEPROM with valid default values
;               initiate save and wait until it has completed.               
;
;
;_essRun        EERPOM Save State function 
;               set ess variable == 1 to start save
;               ess == 0 save complete / idle               
;               ess == 2 save in progress
;               (this function calls other sub functions)
;
;
;_readEERPOM    Restore saved sequence state from EEPROM
;               no parameters
;               W destroyed
;               (this function calls other sub functions)
;
;------------------------------------------------
; PWM Driver 
; 
;
_pwmDriveLo     incfsz          pwmramp,F
                goto            _noRolloverLo
                incf            pwmramp,F
                
                movfw           led1pwm
                movwf           led1pwmWkg
                incfsz          led1pwmWkg,W
                bsf             porta,led1

                movfw           led2pwm
                movwf           led2pwmWkg
                incfsz          led3pwmWkg,W
                bsf             porta,led2

                movfw           led3pwm
                movwf           led3pwmWkg
                incfsz          led3pwmWkg,W
                bsf             porta,led3
                
                movfw           led4pwm
                movwf           led4pwmWkg
                incfsz          led4pwmWkg,W
                bsf             porta,led4
                
                movfw           led5pwm
                movwf           led5pwmWkg
                incfsz          led5pwmWkg,W
                bsf             porta,led5

                movfw           led6pwm
                movwf           led6pwmWkg
                incfsz          led6pwmWkg,W
                bsf             portb,led6

                movfw           led7pwm
                movwf           led7pwmWkg
                incfsz          led7pwmWkg,W
                bsf             portb,led7
                
                movfw           led8pwm
                movwf           led8pwmWkg
                incfsz          led8pwmWkg,W
                bsf             portb,led8

                movfw           led9pwm
                movwf           led9pwmWkg
                incfsz          led9pwmWkg,W
                bsf             portb,led9

                movfw           led10pwm
                movwf           led10pwmWkg
                incfsz          led10pwmWkg,W
                bsf             portb,led10

                movfw           led11pwm
                movwf           led11pwmWkg
                incfsz          led11pwmWkg,W
                bsf             portb,led11
                
                movfw           led12pwm
                movwf           led12pwmWkg
                incfsz          led12pwmWkg,W
                bsf             portb,led12

                movfw           led13pwm
                movwf           led13pwmWkg
                incfsz          led13pwmWkg,W
                bsf             portb,led13

                btfsc           INTCON,T0IF
                call            _timerOut

                
_noRolloverLo   incf            led1pwmWkg,F
                skpnz
                bcf             porta,led1

                incf            led2pwmWkg,F
                skpnz
                bcf             porta,led2

                incf            led3pwmWkg,F
                skpnz
                bcf             porta,led3
                
                incf            led4pwmWkg,F
                skpnz
                bcf             porta,led4
                
				incf            led5pwmWkg,F
                skpnz
                bcf             porta,led5

                incf            led6pwmWkg,F
                skpnz
                bcf             portb,led6

                incf            led7pwmWkg,F
                skpnz
                bcf             portb,led7
                
                incf            led8pwmWkg,F
                skpnz
                bcf             portb,led8
                
                incf            led9pwmWkg,F
                skpnz
                bcf             portb,led9

                incf            led10pwmWkg,F
                skpnz
                bcf             portb,led10

                incf            led11pwmWkg,F
                skpnz
                bcf             portb,led11
                
                incf            led12pwmWkg,F
                skpnz
                bcf             portb,led12

                incf            led13pwmWkg,F
                skpnz
                bcf             portb,led13
                goto            _pwmDriveLo
                

_pwmDriveHi     incfsz          pwmramp,F
                goto            _noRolloverHi
                incf            pwmramp,F
                
                movfw           led1pwm
                movwf           led1pwmWkg
                incfsz          led1pwmWkg,W
                bcf             porta,led1

                movfw           led2pwm
                movwf           led2pwmWkg
                incfsz          led2pwmWkg,W
                bcf             porta,led2

                movfw           led3pwm
                movwf           led3pwmWkg
                incfsz          led3pwmWkg,W
                bcf             porta,led3
                
                movfw           led4pwm
                movwf           led4pwmWkg
                incfsz          led4pwmWkg,W
                bcf             porta,led4
                
                movfw           led5pwm
                movwf           led5pwmWkg
                incfsz          led5pwmWkg,W
                bcf             porta,led5

                movfw           led6pwm
                movwf           led6pwmWkg
                incfsz          led6pwmWkg,W
                bcf             portb,led6

                movfw           led7pwm
                movwf           led7pwmWkg
                incfsz          led7pwmWkg,W
                bcf             portb,led7
                
                movfw           led8pwm
                movwf           led8pwmWkg
                incfsz          led8pwmWkg,W
                bcf             portb,led8

                movfw           led9pwm
                movwf           led9pwmWkg
                incfsz          led9pwmWkg,W
                bcf             portb,led9

                movfw           led10pwm
                movwf           led10pwmWkg
                incfsz          led10pwmWkg,W
                bcf             portb,led10

                movfw           led11pwm
                movwf           led11pwmWkg
                incfsz          led11pwmWkg,W
                bcf             portb,led11
                
                movfw           led12pwm
                movwf           led12pwmWkg
                incfsz          led12pwmWkg,W
                bcf             portb,led12

                 movfw           led13pwm
                movwf           led13pwmWkg
                incfsz          led13pwmWkg,W
                bcf             portb,led13               
                btfsc           INTCON,T0IF
                call            _timerOut

                
_noRolloverHi   incf            led1pwmWkg,F
                skpnz
                bsf             porta,led1

                incf            led2pwmWkg,F
                skpnz
                bsf             porta,led2

                incf            led3pwmWkg,F
                skpnz
                bsf             porta,led3
                

                incf            led4pwmWkg,F
                skpnz
                bsf             porta,led4

				incf            led5pwmWkg,F
                skpnz
                bsf             porta,led5

                incf            led6pwmWkg,F
                skpnz
                bsf             portb,led6

                incf            led7pwmWkg,F
                skpnz
                bsf             portb,led7
                

                incf            led8pwmWkg,F
                skpnz
                bsf             portb,led8

				incf            led9pwmWkg,F
                skpnz
                bsf             portb,led9

                incf            led10pwmWkg,F
                skpnz
                bsf             portb,led10

                incf            led11pwmWkg,F
                skpnz
                bsf             portb,led11
                

                incf            led12pwmWkg,F
                skpnz
                bsf             portb,led12
                                
                incf            led13pwmWkg,F
                skpnz
                bsf             portb,led13
                                
                goto            _pwmDriveHi



;----------------------------------------------
; Cycle through single sequence.
;
_seqData        call            _seqLookup      ; read data
                movwf           rate            ; save in rate
                addlw           .1              ; add 1 (test for 255)
                bz              _seqRestart     ; if data was 255 reset seq
                movlw           rate            ; load W with address of rate var
                movwf           FSR             ; write to FSR
                
_seqNxtData     incf            FSR,F           ; increment FSR
                incf            seql,F          ; increment seq pointer low
                skpnz                           ; skip if result != 0
                incf            seqh,f          ; increment seq pointer high
                
                call            _seqLookup      ; read data
                movwf           INDF            ; save via Indirection 
                movlw           led4new            ; load W with address of blue new var
                xorwf           FSR,W           ; test if FSR == address of blue new var
                bnz             _seqNxtData     ; if not read next data byte
                incf            seql,F          ; otherwise increment seq data pointer
                skpnz                           ; ready for next
                incf            seqh,F          ; call to get seq data
                
                return


_seqRestart     movfw           sbl             ; get sequence base low
                movwf           seql            ; save to current sequence low
                movfw           sbh             ; get sequence base high
                movwf           seqh            ; save to current seqence high
                goto            _seqData        ; load start of sequence
                
_seqLookup      movfw           seqh            ; data table lookup
                movwf           PCLATH
                movfw           seql
                movwf           PCL
                
;----------------------------------------------
; Advance to next sequence (cycle)
;
_advSeq         call            _advLookup      
                addlw           0x01
                bz              _advEnd
                movlw           0x06
                addwf           sbl,f
                skpnc
                incf            sbh,f
                goto            _advSeq
                

_advEnd         incf            currentSeqNo,F
                incf            sbl,f
                skpnz
                incf            sbh,f
                call            _advLookup
                addlw           0x01
                skpnz           
                goto            _advResetBase
                movfw           sbl
                movwf           seql
                movfw           sbh
                movwf           seqh
                clrw                            ; return with W = 0
                return
                
                                
_advResetBase   clrf            currentSeqNo
                movlw           HIGH _base
                movwf           sbh
                movwf           seqh
                movlw           LOW _base
                movwf           sbl
                movwf           seql
                iorlw           0xFF            ; return with W != 0
                return

                ;-------------------------
                ; lookup
_advLookup      movfw           sbh
                movwf           PCLATH
                movfw           sbl
                movwf           PCL

;-------------------------------------------
; Fade Direction
; 

_fadeDir        movfw           hold            ; Set hold timer
                movwf           holdTimerHi
                movfw           rate            ; Set fade rate timer
                movwf           fadeTimer       
                bnz             _calcFade       ; if fade rate == 0
                movfw           led1new            ; No fade so load
                movwf           led1pwm            ; pwm variables
                movfw           led2new            ; with new data
                movwf           led2pwm            ; to get instant
                movfw           led3new            ; colour change
                movwf           led3pwm            ; ...
                movfw           led4new            ; 
                movwf           led4pwm            ; ...
                movfw           led5new            ; No fade so load
                movwf           led5pwm            ; pwm variables
                movfw           led6new            ; with new data
                movwf           led6pwm            ; to get instant
                movfw           led7new            ; colour change
                movwf           led7pwm            ; ...
                movfw           led8new            ; 
                movwf           led8pwm            ; ...
                movfw           led9new            ; No fade so load
                movwf           led9pwm            ; pwm variables
                movfw           led10new            ; with new data
                movwf           led10wm            ; to get instant
                movfw           led11new            ; colour change
                movwf           led11pwm            ; ...
                movfw           led12new            ; 
                movwf           led12pwm            ; ...
                movfw           led13new            ; 
                movwf           led13pwm            ; ...
                clrf            fadeflag        ; clear fade flag so code goes straight to hold
                return                          ; return
                

;-------------------------------------------
; Calculate the difference between current and new PWM value for each colour.
; Difference is absolute value (unsigned)
; Set fade direction to +/-1
                
                
_calcFade       movlw           (1<<led1 | 1<<led2 | 1<<led3 | 1<<led4 | 1<<led5 | 1<<led6 | 1<<led7 | 1<<led8 | 1<<led9 | 1<<led10 | 1<<led11 | 1<<led12 | 1<<led13 )
                movwf           fadeflag        ; preset fade flag
                movlw           0x01            ; preset fade direction to +1 for each colour
                movwf           led1dir
                movwf           led2dir
                movwf           led3dir
                movwf           led4dir
                movwf           led5dir
                movwf           led6dir
                movwf           led7dir
                movwf           led8dir
                movwf           led9dir
                movwf           led10dir
                movwf           led11dir
                movwf           led12dir
                movwf           led13dir
                
                movfw           led1pwm
                subwf           led1new,W          ; subtract current from new (W = current - new)
                bc              _led1abs           ; branch if C=1  (current > new)
                decf            led1dir,F          ; rdir is 1, decrement twice
                decf            led1dir,F          ; to make rdir -1.
                sublw           0x00            ; sub W from 0 to make W positive
_led1abs        movwf           led1dif            ; save difference between current and new


                movfw           led2pwm
                subwf           led2new,W          ; subtract current from new (W = current - new)
                bc              _led2abs           ; branch if C=1  (current > new)
                decf            led2dir,F          ; rdir is 1, decrement twice
                decf            led2dir,F          ; to make rdir -1.
                sublw           0x00            ; sub W from 0 to make W positive
_led2abs        movwf           led2dif            ; save difference between current and new

                movfw           led3pwm
                subwf           led3new,W          ; subtract current from new (W = current - new)
                bc              _led3abs           ; branch if C=1  (current > new)
                decf            led3dir,F          ; rdir is 1, decrement twice
                decf            led3dir,F          ; to make rdir -1.
                sublw           0x00            ; sub W from 0 to make W positive
_led3abs        movwf           led3dif            ; save difference between current and new


                movfw           led4pwm
                subwf           led4new,W          ; subtract current from new (W = current - new)
                bc              _led4abs           ; branch if C=1  (current > new)
                decf            led4dir,F          ; rdir is 1, decrement twice
                decf            led4dir,F          ; to make rdir -1.
                sublw           0x00            ; sub W from 0 to make W positive
_led4abs        movwf           led4dif            ; save difference between current and new

                movfw           led5pwm
                subwf           led5new,W          ; subtract current from new (W = current - new)
                bc              _led5abs           ; branch if C=1  (current > new)
                decf            led5dir,F          ; rdir is 1, decrement twice
                decf            led5dir,F          ; to make rdir -1.
                sublw           0x00            ; sub W from 0 to make W positive
_led5abs        movwf           led5dif            ; save difference between current and new


                movfw           led6pwm
                subwf           led6new,W          ; subtract current from new (W = current - new)
                bc              _led6abs           ; branch if C=1  (current > new)
                decf            led6dir,F          ; rdir is 1, decrement twice
                decf            led6dir,F          ; to make rdir -1.
                sublw           0x00            ; sub W from 0 to make W positive
_led6abs        movwf           led6dif            ; save difference between current and new

                movfw           led7pwm
                subwf           led7new,W          ; subtract current from new (W = current - new)
                bc              _led7abs           ; branch if C=1  (current > new)
                decf            led7dir,F          ; rdir is 1, decrement twice
                decf            led7dir,F          ; to make rdir -1.
                sublw           0x00            ; sub W from 0 to make W positive
_led7abs        movwf           led7dif            ; save difference between current and new


                movfw           led8pwm
                subwf           led8new,W          ; subtract current from new (W = current - new)
                bc              _led8abs           ; branch if C=1  (current > new)
                decf            led8dir,F          ; rdir is 1, decrement twice
                decf            led8dir,F          ; to make rdir -1.
                sublw           0x00            ; sub W from 0 to make W positive
_led8abs        movwf           led8dif            ; save difference between current and new

                movfw           led9pwm
                subwf           led9new,W          ; subtract current from new (W = current - new)
                bc              _led9abs           ; branch if C=1  (current > new)
                decf            led9dir,F          ; rdir is 1, decrement twice
                decf            led9dir,F          ; to make rdir -1.
                sublw           0x00            ; sub W from 0 to make W positive
_led9abs        movwf           led9dif            ; save difference between current and new


                movfw           led10pwm
                subwf           led10new,W          ; subtract current from new (W = current - new)
                bc              _led10abs           ; branch if C=1  (current > new)
                decf            led10dir,F          ; rdir is 1, decrement twice
                decf            led10dir,F          ; to make rdir -1.
                sublw           0x00            ; sub W from 0 to make W positive
_led10abs       movwf           led10dif            ; save difference between current and new

                movfw           led11pwm
                subwf           led11new,W          ; subtract current from new (W = current - new)
                bc              _led11abs           ; branch if C=1  (current > new)
                decf            led11dir,F          ; rdir is 1, decrement twice
                decf            led11dir,F          ; to make rdir -1.
                sublw           0x00            ; sub W from 0 to make W positive
_led11abs       movwf           led11dif            ; save difference between current and new


                movfw           led12pwm
                subwf           led12new,W          ; subtract current from new (W = current - new)
                bc              _led12abs           ; branch if C=1  (current > new)
                decf            led12dir,F          ; rdir is 1, decrement twice
                decf            led12dir,F          ; to make rdir -1.
                sublw           0x00            ; sub W from 0 to make W positive
_led12abs       movwf           led12dif            ; save difference between current and new

                movfw           led13pwm
                subwf           led13new,W          ; subtract current from new (W = current - new)
                bc              _led13abs           ; branch if C=1  (current > new)
                decf            led13dir,F          ; rdir is 1, decrement twice
                decf            led13dir,F          ; to make rdir -1.
                sublw           0x00            ; sub W from 0 to make W positive
_led13abs       movwf           led13dif            ; save difference between current and new

;-------------------------------------------
; Find largest difference between current and new RGBY values
;               
                
	 ; compare red with green difference to find largest
_calcLargest    movfw           led1dif
                subwf           led2dif,W
                movfw	    led2dif                
                skpc            
                movfw	    led1dif
                movwf	    led1cnt	; use Rcnt as temporay variable
                       
                
                ; compare blue with yellow difference to find largest
                movfw           led3dif
                subwf           led4dif,W
                movfw	    led4dif                
                skpc            
                movfw	    led3dif
                movwf	    led3cnt	; use Bcnt as temporay variable
                
                
                ; find larger of previous two comparisons
                movfw           led1cnt
                subwf           led3cnt,W
                movfw           led3cnt
                skpc            
                movfw           led1cnt


_done           sublw           0x00	; make 2's compliment of value in Wreg
                movwf           CntBase
                movwf           led1cnt
                movwf           led2cnt
                movwf           led3cnt
                movwf           led4cnt
                movwf           led5cnt
                movwf           led6cnt
                movwf           led7cnt
                movwf           led8cnt
                movwf           led9cnt
                movwf           led10cnt
                movwf           led11cnt
                movwf           led12cnt
                movwf           led13cnt
                
                return
              


;-------------------------------------------
; Put PIC to sleep
; 
_sleep          movlw           .1
                movwf           ess             ; force EEPROM save to state to initialise
                call            _initLoop       ; save to EERPROM
 	  movfw	    driveLevel
	  movwf           porta	      ; turn off all outputs
                
_waitSwRelSleep btfss           porta,seqSw      ; test seqSw
                goto            _waitSwRelSleep ; wait for it to be released
                movlw           .15
                call            _swDelay
                                
                movfw           porta           ; read GPIO port to clear Int-on-change 
                bcf             INTCON,GPIF     ; clear int-on-change flag
                sleep                           ; put PIC to sleep
                nop

                movlw           .90             ; approx. 1 second delay
                call            _swDelay        ; wait after wake up
                btfsc           porta,seqSw      ; test switch to see if it is still down
                goto            _waitSwRelSleep ; if not go back to sleep
_waitSwRelWake  btfss           porta,seqSw      ; wait for sequence switch to be 
                goto            _waitSwRelWake  ; released before continuing
                goto            _startup        ; After wake-up run startup code
                                                ; as if it was a power-on startup


_swDelay        clrf            TMR0            ; delay to debounce the switch
                bcf             INTCON,T0IF     ; 
_waitSleep      btfss           INTCON,T0IF     ; 
                goto            _waitSleep
                addlw           -.1
                skpz
                goto            _swDelay
                return

; *********************************
; EEPROM write
; call with W reg containg sequence # to save
;
_eeWrite        setbank1
                movwf           EEDATA          ; save W into EEDATA
                incf            EEADR,F
                bsf             EECON1,WREN     ; initiate EEPROM write sequence
                movlw           0x55
                movwf           EECON2
                movlw           0xAA
                movwf           EECON2
                bsf             EECON1,WR
                setbank0
                return                          ; return
              
; *********************************
; EEPROM read
; returns with saved sequence # in W reg

_eeRead         setbank1
	  incf            EEADR,F
                bsf             EECON1,RD       ; initiate EEPROM read
                movf            EEDATA,W        ; load EEPROM data into W
	  setbank0
                return                          ; return   



; *********************************
; Save state to EEPROM 
;
_initEEPROM     incf            ess,F
                clrf            savedSeqNo
                clrf            currentSeqNo
                clrf            freeze
                clrf            led1pwm
                clrf            led2pwm            
                clrf            led3pwm
                clrf            led4pwm
                clrf            led5pwm
                clrf            led6pwm            
                clrf            led7pwm
                clrf            led8pwm
                clrf            led9pwm
                clrf            led10pwm            
                clrf            led11pwm
                clrf            led12pwm
                clrf            led13pwm
_initLoop       call            _essRun
                movf            ess,F
                bnz             _initLoop
                return


_essRun         movfw           ess             ; load state into W reg
                skpnz                           ; skip next if ess != 0
                return                          ; return (ess idle state)
                addlw           -.1             ; test if state == 1
                bz              _essInit        ; run _essInit function if state == 1
                                                ; else drop through and run _essSave
                ; ess 2
                ; 
_essSave        setbank1
                movfw           EECON1          ; read EECON1 register into W
                setbank0
                andlw           (1<<WR)         ; test EEPROM write flag
                skpz                            ; skip if write complete
                return                          ; return if not
                movfw           INDF            ; get data to write using indirect address

                call            _eeWrite        ; start write
                incf            FSR,F           ; increment indirect address pointer
                movfw           FSR             ; 
                xorlw           (base+.7)       ; test if all data written
                skpnz                           ; skip next if not complete
                clrf            ess             ; else reset state variable to 0 (idle)
                return                          ; and return


                ; ess 1
                ; copy variables to save into buffer area
_essInit        movfw           currentSeqNo    ; get variable to save
                movwf           base            ; write to base address
                movwf           base+.6         ; base+6 holds checksum
                
                movfw           led1pwm            ; get next variable to save
                movwf           base+.1         ; write to base+1
                addwf           base+.6,F       ; add to checksum
                
                movfw           led2pwm            ; et al.
                movwf           base+.2
                addwf           base+.6,F       ; add to checksum        
                
                movfw           led3pwm
                movwf           base+.3
                addwf           base+.6,F       ; add to checksum
                
                movfw           led4pwm
                movwf           base+.4
                addwf           base+.6,F       ; add to checksum

                movfw           led5pwm            ; get next variable to save
                movwf           base+.1         ; write to base+1
                addwf           base+.6,F       ; add to checksum
                
                movfw           led6pwm            ; et al.
                movwf           base+.2
                addwf           base+.6,F       ; add to checksum        
                
                movfw           led7pwm
                movwf           base+.3
                addwf           base+.6,F       ; add to checksum
                
                movfw           led8pwm
                movwf           base+.4
                addwf           base+.6,F       ; add to checksum

                movfw           led9pwm            ; get next variable to save
                movwf           base+.1         ; write to base+1
                addwf           base+.6,F       ; add to checksum
                
                movfw           led10pwm            ; et al.
                movwf           base+.2
                addwf           base+.6,F       ; add to checksum        
                
                movfw           led11pwm
                movwf           base+.3
                addwf           base+.6,F       ; add to checksum
                
                movfw           led12pwm
                movwf           base+.4
                addwf           base+.6,F       ; add to checksum

                movfw           led13pwm
                movwf           base+.4
                addwf           base+.6,F       ; add to checksum
                
                movfw           freeze
                movwf           base+.5
                addwf           base+.6,W       ; add to checksum
                sublw           0x00
                movwf           base+.6
                                
                movlw           base            ; load W with address of base
                movwf           FSR             ; write to FSR indirect register
                setbank1
                clrf            EEADR           ; set EEADR to 0
                setbank0                
                incf            ess,F           ; set to next state
                return

; *********************************
; Swap FSR with FSRsave;

_swapFSR        movfw           FSR
                xorwf           FSRsave,F
                xorwf           FSRsave,W
                xorwf           FSRsave,F
                movwf           FSR
                return          


; *********************************
; Restore state from EEPROM 
;
_readEEPROM     setbank1
                btfsc           EECON1,WR
                goto            $-1
                clrf            EEADR
                call            _eeRead
                movwf           savedSeqNo
                movwf           saveCnt
                
                call            _eeRead
                movwf           led1pwm
                addwf           saveCnt,F
                
                call            _eeRead
                movwf           led2pwm
                addwf           saveCnt,F

                call            _eeRead
                movwf           led3pwm
                addwf           saveCnt,F

                call            _eeRead
                movwf           led4pwm
                addwf           saveCnt,F

                call            _eeRead
                movwf           led5pwm
                addwf           saveCnt,F
                
                call            _eeRead
                movwf           led6pwm
                addwf           saveCnt,F

                call            _eeRead
                movwf           led7pwm
                addwf           saveCnt,F

                call            _eeRead
                movwf           led8pwm
                addwf           saveCnt,F

                call            _eeRead
                movwf           led9pwm
                addwf           saveCnt,F
                
                call            _eeRead
                movwf           led10pwm
                addwf           saveCnt,F

                call            _eeRead
                movwf           led11pwm
                addwf           saveCnt,F

                call            _eeRead
                movwf           led12pwm
                addwf           saveCnt,F

                call            _eeRead
                movwf           led13pwm
                addwf           saveCnt,F

                call            _eeRead
                movwf           freeze
                addwf           saveCnt,F

                call            _eeRead
                addwf           saveCnt,W       ; returns with checksum in W
                return