;--------------------------------------------------------
; 5mS Time Out
; This code runs approximately every 5mS
; The PWM code calls it at the end of each PWM cycle if TMR0 has
; overflowed.  Because the PWM code takes approximatley 4mS to run
; we add the current TMR0 contents to the cTIMER constant so that
; over a number of passes the average time-out is 5mS
; This method is accurate enough to get predicatable delays
; needed for this application.  However because the write to the timer
; clears the prescaler at some random point, it isn't accurate over
; many seconds or minutes.
 

_timerOut       movfw           TMR0            ; load current TMR0 into W
                addlw           cTIMER          ; add TIMER constant to W
                movwf           TMR0            ; write back to TMR0
                bcf             INTCON,T0IF     ; clear Timer 0 Interrupt flag
	  ;return ; *** DEBUG only***

;-----------------------------------------------------
; EEPROM save counter
                                
                call            _swapFSR
                call            _essRun
                call            _swapFSR
                
                movf            saveCnt,F
                bz              _switchCheck
                decfsz          saveCntPS,F
                goto            _switchCheck
                decfsz          saveCnt,F
                goto            _switchCheck
                movlw           .1
                movwf           ess
;-----------------------------------------------------
; Sequence Switch Check

_switchCheck    movf            seqSwDblClk,F
                ;skpz
              ; decf            seqSwDblClk,F   

                btfsc           porta,seqSw
                goto            _seqSwUp        
                                
_seqSwDown      incf            seqSwCnt,F      ; increment seqSwCnt counter variable
                skpnz                           ; if result == 0, counter has wrapped round
                goto            _sleep          ; so goto sleep
                goto            _fadeState

_seqSwUp        movlw           -.4             ; 20mS
                addwf           seqSwCnt,W      ; was switch down for over 20mS
                bc              _swShort
                clrf            seqSwCnt
                goto            _fadeState
                

_swShort        movf            seqSwDblClk,F
               bnz             _swDoubleClick
                
                clrf            seqSwCnt
                movlw           0x01
                xorwf           freeze,F
                movlw           .40
               movwf           seqSwDblClk
                movlw           cSAVETIMER      ; set save to EEPROM timer
                movwf           saveCnt         ; for delay after last sequence change.
                goto            _fadeState


_swDoubleClick  call            _advSeq         ; advance to next sequence
               call            _seqData        ; get first set of sequence data
                clrf            led1pwm            ; reset current led1 pwm to 0
                clrf            led1pwm            ; reset current led2 pwm to 0
                clrf            led3pwm            ; reset current led3 pwm to 0
                clrf            led4pwm            ; reset current led4 pwm to 0
                clrf            led5pwm            ; reset current led5 pwm to 0
                clrf            led6pwm            ; reset current led6 pwm to 0
                clrf            led7pwm            ; reset current led7 pwm to 0
                clrf            led8pwm            ; reset current led8 pwm to 0
                clrf            led9pwm            ; reset current led9 pwm to 0
                clrf            led10pwm           ; reset current led10 pwm to 0
                clrf            led11pwm           ; reset current led11 pwm to 0
                clrf            led12pwm           ; reset current led12 pwm to 0
                clrf            led13pwm           ; reset current led13 pwm to 0

                call            _fadeDir        ; set fade direction
                movlw           cHOLDPRESCALER  ; reset hold time prescaler
                movwf           holdTimerPS
                clrf            seqSwCnt
               movlw           cSAVETIMER      ; set save to EEPROM timer
               movwf           saveCnt         ; for delay after last sequence change.
              ; clrf            seqSwDblClk
                clrf            freeze

                                

;-----------------------------------------------------
; fade / hold state function select
                
_fadeState      btfsc           freeze,0        
                return

                movf            fadeflag,F
                bz              _holding
;------------------------------------------------------
; fade          
                
_fading         decfsz          fadeTimer,F
                return
                movfw           rate
                movwf           fadeTimer

;----------------------------------------
; Fade stepper
;

_led1fade          movfw           led1new
                xorwf           led1pwm,W
                bnz             $+3
                bcf             fadeflag,led1
                goto            _led2fade

                movfw           led1dif
                addwf           led1cnt,F
                skpc            
                goto            _led2fade
                movfw           CntBase
                addwf           led1cnt,F
                movfw           led1dir
                addwf           led1pwm,F
                
_led2fade       movfw           led2new
                xorwf           led2pwm,W
                bnz             $+3
                bcf             fadeflag,led2
                goto            _led3fade

                movfw           led2dif
                addwf           led2cnt,F
                skpc            
                goto            _led3fade
                movfw           CntBase
                addwf           led2cnt,F
                movfw           led2dir
                addwf           led2pwm,F

_led3fade          movfw           led3new
                xorwf           led3pwm,W
                bnz             $+3
                bcf             fadeflag,led3
	            goto	    _led4fade
                 
                movfw           led3dif
                addwf           led3cnt,F
                skpc            
                goto            _led4fade
                movfw           CntBase
                addwf           led3cnt,F
                movfw           led3dir
                addwf           led3pwm,F
                
                
_led4fade       movfw           led4new
                xorwf           led4pwm,W
                bnz             $+3
                bcf             fadeflag,led4
                return
                
                movfw           led4dif
                addwf           led4cnt,F
                skpc            
				goto            _led5fade          
                movfw           CntBase
                addwf           led4cnt,F
                movfw           led4dir
                addwf           led4pwm,F

led5fade        movfw           led5new
                xorwf           led5pwm,W
                bnz             $+3
                bcf             fadeflag,led5
                goto            _led6fade

                movfw           led5dif
                addwf           led5cnt,F
                skpc            
                goto            _led6fade
                movfw           CntBase
                addwf           led5cnt,F
                movfw           led5dir
                addwf           led5pwm,F
                
_led6fade       movfw           led6new
                xorwf           led6pwm,W
                bnz             $+3
                bcf             fadeflag,led6
                goto            _led7fade

                movfw           led6dif
                addwf           led6cnt,F
                skpc            
                goto            _led7fade
                movfw           CntBase
                addwf           led6cnt,F
                movfw           led6dir
                addwf           led6pwm,F

_led7fade       movfw           led7new
                xorwf           led7pwm,W
                bnz             $+3
                bcf             fadeflag,led7
	            goto	    _led8fade
                
                movfw           led7dif
                addwf           led7cnt,F
                skpc            
                goto            _led8fade
                movfw           CntBase
                addwf           led7cnt,F
                movfw           led7dir
                addwf           led7pwm,F
                
                
_led8fade       movfw           led8new
                xorwf           led8pwm,W
                bnz             $+3
                bcf             fadeflag,led8
                return
                goto            _led9fade 
                movfw           led8dif
                addwf           led8cnt,F
                skpc            
                goto            _led9fade   
                movfw           CntBase
                addwf           led8cnt,F
                movfw           led8dir
                addwf           led8pwm,F

led9fade          movfw           led9new
                xorwf           led9pwm,W
                bnz             $+3
                bcf             fadeflag,led9
                goto            _led10fade

                movfw           led9dif
                addwf           led9cnt,F
                skpc            
                goto            _led10fade
                movfw           CntBase
                addwf           led9cnt,F
                movfw           led9dir
                addwf           led9pwm,F
                
_led10fade       movfw           led10new
                xorwf           led10pwm,W
                bnz             $+3
                bcf             fadeflag,led10
                goto            _led11fade

                movfw           led10dif
                addwf           led10cnt,F
                skpc            
                goto            _led11fade
                movfw           CntBase
                addwf           led10cnt,F
                movfw           led10dir
                addwf           led10pwm,F

_led11fade      movfw           led11new
                xorwf           led11pwm,W
                bnz             $+3
                bcf             fadeflag,led11
	            goto	    _led12fade
                
                movfw           led11dif
                addwf           led11cnt,F
                skpc            
                goto            _led12fade
                movfw           CntBase
                addwf           led11cnt,F
                movfw           led11dir
                addwf           led11pwm,F
                
                
_led12fade      movfw           led12new
                xorwf           led12pwm,W
                bnz             $+3
                bcf             fadeflag,led12
				goto            _led13fade
                
                movfw           led12dif
                addwf           led12cnt,F
                skpc            
				goto            _led13fade      
                movfw           CntBase
                addwf           led12cnt,F
                movfw           led12dir
                addwf           led12pwm,F

_led13fade      movfw           led13new
                xorwf           led13pwm,W
                bnz             $+3
                bcf             fadeflag,led13
			return
                
                movfw           led13dif
                addwf           led13cnt,F
                skpc            
		return   
                movfw           CntBase
                addwf           led13cnt,F
                movfw           led13dir
                addwf           led13pwm,F

                return                




;------------------------------------------------------
; Hold pwm values timer         
                
_holding        movf            holdTimerHi,F   ; test holdTimerHi == 0
                bz              _holdSeqNext    ; no hold if it was 0
                
                decfsz          holdTimerPS,F
                return
                movlw           cHOLDPRESCALER
                movwf           holdTimerPS
                decfsz          holdTimerHi,F
                return
_holdSeqNext    call            _seqData
                call            _fadeDir
                
                return

